// *****************************************************************************
// name: swordSimul.sce
//
// Example file showing how to simulate blade properties.
//
// Copyright (C) 2007 Vincent Le Chevalier
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 51
// Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// *****************************************************************************


mode(0); // supress the echo to the scilab console
clear; // clear the environment

exec("swordSimulFunc.sce"); // load the simulation functions

// This function is a shortcut for defining a cross-section. It allows to
// specify a diamond cross-section with a fuller that is also diamond shaped.
// 0 < w < 1 : relative width of the fuller
// 0 < d < 1 : relative depth of the fuller
function geom = fulleredDiamond(w, d)
    geom = fromQuarterSection([[0 w 1];[(1-d) 1 0]]);
endfunction;

// A simple square cross-section
squareSec = fromQuarterSection([[0 1];[1 1]]);

// Caracteristics of the steel: rho0 is the density, E0 is the Young modulus.
// This density was computed from the mass of the sword and the measurements.
// It is actually a bit high, which may indicate that my reconstruction is imperfect.
rho0 = 8334.0595;
//E0 = 300e9;
E0 = 250e9;

// Measurements of the sword.
// These are actual measurements on an Angus Trim Type XI sword that I bought in
// 2001 from Albion Armorers.
//
// We store the measurements as a big table where each line contains:
// - the position on the sword
// - the width of the sword (profile taper)
// - the thickness of the sword (distal taper)
// - the number of finite elements that should be built between this position
// and the next one (use 0 when you want to introduce discontinuities)
// - the caracteristics of the section that is used for these elements (actually
// a table of 3 numbers)
// - the density
// - the Young modulus
// All the distances are in meters.
measures = [
// Pommel
// A squarish pommel is used here, because the shape is not really important,
// it's the mass that counts.
[0     0.04 0.011 1 squareSec rho0 E0];
[0.04  0.04 0.011 0 squareSec rho0 E0];
// Comment the two previous lines and uncomment the next one to obtain a sword
// without pommel.
//[0     0.0059 0.0059 1 squareSec rho0 E0];
// Tang
[0.04  0.0059 0.0059 5 squareSec rho0 E0];
[0.137 0.02   0.0059 0 squareSec rho0 E0];

// Cross-guard
[0.137 0.22 0.01 1 squareSec rho0 E0];
[0.147 0.22 0.01 0 squareSec rho0 E0];
// Comment the two previous lines and uncomment these two next lines to simulate
// a sword without the cross.
//[0.137 0.02 0.0059 1 squareSec rho0 E0];
//[0.147 0.02 0.0059 0 squareSec rho0 E0];

// Fuller
// It has a constant width.
[0.147 38.5e-3 5.9e-3 30 fulleredDiamond(10/38.5, 2.4/5.9) rho0 E0];
[0.605 36.9e-3 4e-3   10 fulleredDiamond(10/36.9, 2.4/5.9) rho0 E0];
[0.78  34.5e-3 3.4e-3  0 squareSec rho0 E0];
//Tip
[0.78  34.4e-3 3.5e-3  5 fulleredDiamond(10/34.5, 0) rho0 E0];
[0.925 31.8e-3 2.9e-3  1 fulleredDiamond(10/34.5, 0) rho0 E0];
[0.98    3e-3 1.4e-3  0 squareSec rho0 E0];
[0.98    0 0  0 squareSec rho0 E0];
];

//For a simple stick
//measures = [
//[0 0.04 0.011 30 squareSec rho0 E0];
//[1 0.04 0.011 0 squareSec rho0 E0];
//];

// The measured mass can be specified.
// In this case, rho0 is adjusted so that the simulated sword will have the
// exact same mass as the real one.
// When measuredMass is negative it is not taken into account.
measuredMass = -1.009;
// The number of modes we want to see.
nmodes = 5;
// Aft and forward positions on the grip.
aft = 0.043; forward = 0.147;

// reference point for the grip (junction cross-handle)
reference = 0.135;

// Variable used when plotting the modes. We use 800 points spaced regularly
// over the length of the sword.
plotx = linspace(measures(1,1), measures($,1), 800);

// Build the actual finite-element representation.
// x0  : where the elements start and end
// rho : the density of each element
// S   : the surface of the cross-section of each elements
// E   : the Young modulus for each element
// Iy  : the area moment of inertia for each direction, in flat-to-flat direction
// Iz  : the area moment of inertia for each direction, in edge-to-edge direction
[x0, rho, S, E, Iy, Iz] = buildElements(measures);

// Get the dynamic properties (mass, center of gravity, and the square of the
// radius of gyration).
[Mass, G, k2] = dynamicProperties(x0, rho, S);
if measuredMass > 0 then
    rho = rho * measuredMass/Mass; // Mass correction
    [Mass, G, k2] = dynamicProperties(x0, rho, S); // Recompute with the new rho
end;
rhoAve = mean(rho);
printf("Steel density: %.2f kg.m^-3\n", rhoAve);

// Computation of the pivot points.
aftpivot = pivotPoint(aft, G, k2);
forwardpivot = pivotPoint(forward, G, k2);

// Build the mass matrix from the finite elements.
M=buildMassMatrix(x0, rho, S);

// Build two stiffness matrices, for the two planes of vibration.
Ky = buildStiffnessMatrix(x0, E, Iy);
Kz = buildStiffnessMatrix(x0, E, Iz);

// Compute modes of vibration
[modesy, freqsy, nFinaly] = getModes(M, Ky, nmodes);
[modesz, freqsz, nFinalz] = getModes(M, Kz, nmodes);

// Get the nodes
function bladeNode = getBladeNode(modes)
    mode1 = modes(:,1);
    nodesVal = nodes(x0, mode1);
    bladeNode = max(nodesVal); 
endfunction;

function handNode = getHandNode(modes)
    mode1 = modes(:,1);
    nodesVal = nodes(x0, mode1);
    handNode = min(nodesVal); 
endfunction;

node1 = getHandNode(modesy);
node2 = getBladeNode(modesy);

// write data in a file readable in the Weapon Dynamics Computer
fd = mopen("simulated.json", 'w');
mfprintf(fd,"{""weapons"":{""simulated"":{\n");
mfprintf(fd,"   ""M"":%.2f, ""CoG"":%.2f, ""K"": %f,\n", Mass*1000, G*100, k2*10000);
mfprintf(fd,"   ""butt"":%.2f, ""R"":%.2f, ""tip"":%.2f,\n", measures(1,1)*100, reference*100, measures($,1)*100);
mfprintf(fd,"   ""node1"":%.2f,""node2"":%.2f,\n", node1*100, node2*100);
mfprintf(fd,"   ""p0s"":[%.2f, %.2f],""p1s"":[%.2f, %.2f],\n", aft*100, forward*100, aftpivot*100, forwardpivot*100);
mfprintf(fd,"   ""dK"":0,""lever"":null,""m"":""w"",""conf"":[1,1],""Np"":null,""pps"":[null],""Ts"":[null]}},""version"":""2""\n}\n");
mclose(fd);

printf("Mass = %.2fg, CoG = %.2fcm,\n aft pair = %.2fcm, %.2fcm\n forward pair = %.2fcm, %.2fcm\n nodes = %.2fcm, %.2fcm\n", ...
    Mass*1000, G*100, aft*100, aftpivot*100, forward*100, forwardpivot*100, node1*100, node2*100);

function plotHarmonics(modes, freqs, nFinal)
    plot([0; x0($)], [0; 0], 'k');
    bladeNode = getBladeNode(modes);
    handNode = getHandNode(modes);
    y = [];
    for i=1:nFinal;
        vec = modes(:,i);
        nodesVal = gsort(nodes(x0, vec), 'g', 'i');
        printf("mode %i:\n f = %f\n", i, freqs(i));
        printf(" nodes = [ ");
        for j=1:size(nodesVal,'*');
            printf("%f ", nodesVal(j));
        end;
        printf("]\n");
        y = [y plotVal(x0, vec, plotx)];
    end;
    plot2d(plotx, y);
    plot2d(plotx, -y);
endfunction;

function plotWeaponDyn()
    plot(G, 0, 'kX', "MarkerSize", 10);
    plot([aft, aftpivot], [0, 0], 'bD', "MarkerSize", 10);
    plot([forward, forwardpivot], [0, 0], 'rD', "MarkerSize", 10);
endfunction;

// Plot the measures
scf(0);clf;
fig = gcf();
fig.figure_size = [1024, 768];
subplot(2,1,1);
plot(measures(:,1), measures(:,2)/2 * [1 -1], 'b');
plotWeaponDyn();
a=gca();a.isoview = "on";
a=gca(); a.title.text = "Width";
a.font_size = 2;
a.title.font_size = 4;
subplot(2,1,2);
plot(measures(:,1), measures(:,3)/2 * [1 -1], 'b');
a=gca(); a.title.text = "Thickness";
a.font_size = 2;
a.title.font_size = 4;
//a=gca();a.isoview = "on";

// Plot the linear density (mass distribution)
scf(1);clf;
fig = gcf();
fig.figure_size = [1024, 768];
//plot(x0(1:$-1), sqrt(1/rhoAve/%pi*rho.*S)');
toplot = rho.*S;
plot(x0(1:$), [toplot, toplot($)]);
a=gca();
poly1= a.children.children(1);
poly1.polyline_style = 2;
a.title.text = "Mass distribution";
a.auto_scale = "off";
a.data_bounds=[x0(1), 0; x0($), mean(toplot)*1.05];  
a.font_size = 2;
a.title.font_size = 4;

printf("************\nFlat to flat vibrations\n************\n");
scf(2);clf();
plotHarmonics(modesy, freqsy, nFinaly);
plotWeaponDyn();
fig = gcf();
fig.figure_size = [1024, 768];
a = gca();
a.title.text = "Flat to flat";
a.font_size = 2;
a.title.font_size = 4;

printf("************\nEdge to edge vibrations\n************\n");
scf(3);clf();
plotHarmonics(modesz, freqsz, nFinalz);
plotWeaponDyn();
fig = gcf();
fig.figure_size = [1024, 768];
a = gca();
a.title.text = "Edge to edge";
a.font_size = 2;
a.title.font_size = 4;
