// name: swordSimulFunc.sce
//
// Functions allowing to simulate blade properties.
//
// Copyright (C) 2007 Vincent Le Chevalier
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

function geom = fromQuarterSection(section)
//(%i12) factor(integrate(integrate(1, y, 0, (y1-y0)*(z-z0)/(z1-z0)+y0), z, z0, z1));
//                              (y1 + y0) (z1 - z0)
//(%o12)                        -------------------
//                                       2
//(%i13) factor(integrate(integrate(y^2, y, 0, (y1-y0)*(z-z0)/(z1-z0)+y0), z, z0, z1));
//                                     2     2
//                        (y1 + y0) (y1  + y0 ) (z1 - z0)
//(%o13)                  -------------------------------
//                                      12
//(%i16) factor(integrate(z^2*((y1-y0)*(z-z0)/(z1-z0)+y0), z, z0, z1));
//(%o16) 
//                      2        2                                  2          2
//    (z1 - z0) (3 y1 z1  + y0 z1  + 2 y1 z0 z1 + 2 y0 z0 z1 + y1 z0  + 3 y0 z0 )
//    ---------------------------------------------------------------------------
//                                        12
    geom = [0 0 0];
    zmin = section(1,1);
    zmax = section(1,$);
    ymax = max(section(2,:));
    ymin = 0;//min(section(2,:));
    
    section(1,:) = (section(1,:) - zmin) / (zmax - zmin);
    section(2,:) = (section(2,:) - ymin) / (ymax - ymin);

    for i=1:(size(section,'c')-1);
        z0 = section(1, i);
        z1 = section(1, i+1);
        y0 = section(2, i);
        y1 = section(2, i+1);
        geom(1) = geom(1) + (y1+y0)*(z1-z0)/2;
        geom(2) = geom(2) + (y1+y0)*(y1^2+y0^2)*(z1-z0)/12;
        geom(3) = geom(3) + (...
                    3*y1*z1^2 + y0*z1^2 + 2*y1*z0*z1 + 2*y0*z0*z1 ...
                    + y1 * z0^2 + 3 * y0 * z0^2 ...
                )*(z1-z0)/12;
    end; 

    geom(2) = geom(2) * 3;
    geom(3) = geom(3) * 3;
    
endfunction;

function [x0, rho, S, E, Iy, Iz] = buildElements(measures)
    x0 = [measures(1,1)];
    E = [];
    Iy = [];
    Iz = [];
    S = [];
    rho = [];
    Nsec = size(measures, 'r') - 1;

    for i=1:Nsec;
        N = measures(i,4);
        if N <> 0 then
            xStart = measures(i,1);
            xEnd = measures(i+1,1);
            xAdd = linspace(xStart, xEnd, N+1);

            x0 = [x0 xAdd(2:$)];
            prof = linspace(measures(i,2), measures(i+1,2), N+1);
            dist = linspace(measures(i,3), measures(i+1,3), N+1);
            prof = (prof(1:N) + prof(2:N+1))/2;
            dist = (dist(1:N) + dist(2:N+1))/2;
            E = [E ones(1,N)*E0];
            rho = [rho ones(1,N)*rho0];
            geom = measures(i,5:7);
            S = [S (geom(1) .* prof .* dist)];
            Iy = [Iy (geom(2)/12 .* prof .* dist .^ 3)];
            Iz = [Iz (geom(3)/12 .* prof .^ 3 .* dist)];

        end;
    end;
 
endfunction;

function [Mass, G, k2] = dynamicProperties(x0, rho, S)

    N = size(x0, 'c') - 1;
    dl=x0(2:N+1) - x0(1:N); 

    Mass=sum(rho .* S .* dl);
    G=sum((x0((1:N)+1)+x0(1:N))/2 .* rho .* S .* dl)/Mass;
    k2=sum( (((x0((1:N)+1)+x0(1:N))/2 - G).^2 + dl.^2/12) .* rho .* S .* dl)/Mass;
endfunction;

function p2 = pivotPoint(p1, g, k2)
    p2 = g - k2/(p1-g);
endfunction;

function M = buildMassMatrix(x0, rho, S)
    N = size(x0, 'c') - 1;
    dl=x0(2:N+1) - x0(1:N); 
    dof = 2*N + 2;
    
    M = zeros(dof,dof);
    
    M0 =[
    [ 13/35  11/210   9/70  -13/420 ]
    [ 11/210  1/105  13/420  -1/140 ]
    [  9/70  13/420  13/35  -11/210 ]
    [-13/420 -1/140 -11/210   1/105 ]
    ];

    for i=1:N;
        dli = dl(i);
        W = dli * [1 dli 1 dli]' * [1 dli 1 dli];
        ind = 2*i-1;
        sub = ind:ind+3;
        chunk = rho(i)*S(i)*(W.*M0);
        M(sub,sub) = M(sub,sub) + chunk;
    end;    
endfunction;

function K = buildStiffnessMatrix(x0, E, I)
    N = size(x0, 'c') - 1;
    dl=x0(2:N+1) - x0(1:N); 
    dof = 2*N + 2;
    
    K = zeros(dof,dof);
    
    K0 =[
    [ 12  6 -12  6 ]
    [  6  4  -6  2 ]
    [-12 -6  12 -6 ]
    [  6  2  -6  4 ]
    ];

    for i=1:N;
        dli = dl(i);
        W = 1/dli**3 * [1 dli 1 dli]' * [1 dli 1 dli];
        ind = 2*i-1;
        sub = ind:ind+3;
        chunk = E(i)*I(i)*(W.*K0);
        K(sub,sub) = K(sub,sub) + chunk;    
    end;    
endfunction;

function [modes, freqs, n] = getModes(M, K, nModes)
    [vecs, vals] = spec(M^(-1)*K);
    vals = diag(vals);
    [ovals,order] = gsort(abs(vals), 'g', 'i');
    ovecs = vecs(:,order);
    
    endMode = min(nModes+2, size(ovecs, 'c'));
    modes = ovecs(:,3:endMode);
    freqs = sqrt(ovals(3:endMode))/(2*%pi);
    scal = 1/2*abs(modes' * K * modes);
    //print(%io(2),scal)
    scal = diag(diag(scal));
    //print(%io(2),scal)
    modes = modes/(scal^0.5);
    n = endMode - 2;
endfunction;

function [sol]=nodes(x0, u)
    N = size(x0, 'c') - 1;
    dl=x0(2:N+1) - x0(1:N); 
    //dof = 2*N + 2;
    R = [
    [ 1  0 -3  2 ]
    [ 0  1 -2  1 ]
    [ 0  0  3 -2 ]
    [ 0  0 -1  1 ]
    ];
    eps= 1e-10; //Un petit nombre

    sol = [];
    for i=1:N;
        ind = 2*i-1;
        //pause;
        uloc = u(ind:ind+3);

        W = [1 dl(i) 1 dl(i)];
        Coeffs = (uloc' .* W) * R;
        allsol = roots(Coeffs(4:-1:1))
        for j=1:size(allsol,'*');
            val = real(allsol(j));
            if  (val <= 1) & (val >= 0) & abs(imag(allsol(j))) < eps then
                sol = [sol (val*dl(i)+x0(i))];
            end;
        end;
    end;
endfunction;

function y = plotVal(x0, u , xplot)
    y = [];
    N = size(x0, 'c') - 1;
    dl=x0(2:N+1) - x0(1:N); 
    R = [
    [ 1  0 -3  2 ]
    [ 0  1 -2  1 ]
    [ 0  0  3 -2 ]
    [ 0  0 -1  1 ]
    ];
    j = 1;
    for i = 1:N;
        xtemp = [];
        while (j<=size(xplot, '*')) & (xplot(j) <= x0(i+1)) & (xplot(j) >= x0(i));
            xtemp = [xtemp xplot(j)];
            j = j+1;
        end;
        //pause;
        xtemp = (xtemp - x0(i)) / (x0(i+1) - x0(i));
        Xpow = [ones(xtemp); xtemp; xtemp.^2; xtemp.^3];
        ind = 2*i-1;
        uloc = u(ind:ind+3);
        W = [1 dl(i) 1 dl(i)];
        vals = (uloc' .* W) * R * Xpow; 
        
        y = [y; vals'];
    end;
endfunction;


